#include "Exposure.h"
#include <iomanip>
#include <iostream>
#include <map>
#include <Random.h>


Exposure::Exposure(FrameList &frameList)
{
	_frameList = frameList;
}

FrameList Exposure::getFrameList()
{
  return _frameList;
}

void Exposure::setFrameList(FrameList frl)
{
  _frameList = frl;
}


/*
 * Sum of all the fracexp of all the frames inside
 * each light curve gti.
 */
void Exposure::doCorr(LCGrid *lc,double area)
{

  std::vector<double> frameTime, frameTimeNoArea,gtiTime;
	// Initialize _fracExp vector
	_expoCorr =  new std::vector<double>;

	for (unsigned int i = 0 ; i<=lc->_cells ; i++)
	  {
	    _expoCorr->push_back(0.0);
	    frameTime.push_back(0.0);
	    frameTimeNoArea.push_back(0.0);
	    gtiTime.push_back(0.0);
	  }

	unsigned int i = 0;  // GTI counter
	unsigned int n = 0;  // Frames counter


	// Skip all the frames that starts earlier than the first GTI
	while ((_frameList._frameList[n].start() < (*(lc->_gtiList))[0].start()) &&
			( _frameList._frameList[n].stop() < (*(lc->_gtiList))[0].start() ) )
	{
		errstr<< "Frame[ " << n << " ] = [ "<< _frameList._frameList[n].start() << " , "
				<< _frameList._frameList[n].stop() << " ]  SKIPPED"<< Sas::message(Msg::Lib, Msg::Verbose);
		n++;
	}

	while ( ( i != lc->_gtiList->size() ) && (n != _frameList._frameList.size() ) )
	{
		errstr<< std::setprecision(19)<< "GTI[ "<<i<<" ] = [ "<< (*(lc->_gtiList))[i].start() << " , "
			<< (*(lc->_gtiList))[i].stop() << " ]"<< Sas::message(Msg::Lib, Msg::Verbose);
		//double fracExp = 0.0;
		double frameBinTime = 0.0;
		double frameBinTimeNoArea = 0.0;
		bool nextGTI = false;
		while ( (n != _frameList._frameList.size() )
				&& ( nextGTI == false) )
		{
			bool checkFrameAgain = false;
		  errstr<< "Frame[ " << n << " ] = [ "<< _frameList._frameList[n].start() << " , "
			<< _frameList._frameList[n].stop() << " ]"<< Sas::message(Msg::Lib, Msg::Verbose);

			/* AI gcc4 porting
			 * error: suggest explicit braces to avoid ambiguous ‘else’
			 */
			if ( (_frameList._frameList[n].start() <= (*(lc->_gtiList))[i].start()) )
			{
				if ( (_frameList._frameList[n].stop() > (*(lc->_gtiList))[i].start() )
						&&	(_frameList._frameList[n].stop() <= (*(lc->_gtiList))[i].stop() ) )
				{
					errstr << "First frame inside the bin[ "<< n << " ] = [ "<< _frameList._frameList[n].start() << " , "
					<< _frameList._frameList[n].stop() << " ]" << Sas::message(Msg::Lib, Msg::Verbose);

					frameBinTime = frameBinTime +_frameList._frames[n]._fracExp*(_frameList._frameList[n].stop()-(*(lc->_gtiList))[i].start())*area;
					frameBinTimeNoArea = frameBinTimeNoArea +_frameList._frames[n]._fracExp*(_frameList._frameList[n].stop()-(*(lc->_gtiList))[i].start());

				}
				else if (_frameList._frameList[n].stop() >= (*(lc->_gtiList))[i].stop() )
				{
					errstr << "Frame later than GTI bin[ "<< n << " ] = [ "<< _frameList._frameList[n].start() << " , "
					<< _frameList._frameList[n].stop() << " ]" << Sas::message(Msg::Lib, Msg::Verbose);
					//fracExp = ( (lc->_gtiList[i].stop()-lc->_gtiList[i].start())/
					//		(_frameList._frameList[n].stop() - _frameList._frameList[n].start()) )*_frameList._frames[n]._fracExp;

					//frameBinTime = frameBinTime +fracExp*(lc->_gtiList[i].stop()-lc->_gtiList[i].start());
					frameBinTime = frameBinTime +_frameList._frames[n]._fracExp*((*(lc->_gtiList))[i].stop()-(*(lc->_gtiList))[i].start())*area;
					frameBinTimeNoArea = frameBinTimeNoArea +_frameList._frames[n]._fracExp*((*(lc->_gtiList))[i].stop()-(*(lc->_gtiList))[i].start());

					nextGTI = true;
					//Just in case the first frame
					if ( n != 0)
						n--;
					else
						checkFrameAgain =  true;

				}
			}

			if  ( ( _frameList._frameList[n].start() > (*(lc->_gtiList))[i].start() )
					&& ( _frameList._frameList[n].start() < (*(lc->_gtiList))[i].stop() ) )
			{
				if (_frameList._frameList[n].stop() <= (*(lc->_gtiList))[i].stop() )
				{
					errstr << "Fraction of frame " << _frameList._frameList[n].stop() << " " << _frameList._frameList[n].start()
							<< Sas::message(Msg::Lib, Msg::Verbose);

					frameBinTime = frameBinTime +
						_frameList._frames[n]._fracExp*(_frameList._frameList[n].stop()-_frameList._frameList[n].start())*area;
					frameBinTimeNoArea = frameBinTimeNoArea +
						_frameList._frames[n]._fracExp*(_frameList._frameList[n].stop()-_frameList._frameList[n].start());

				}
				else if ( _frameList._frameList[n].stop() > (*(lc->_gtiList))[i].stop())
				{
					errstr << "Last frame in current bin[ "<< n << " ] = [ " << _frameList._frameList[n].start() << " , "
						<< _frameList._frameList[n].stop() << " ]" << Sas::message(Msg::Lib, Msg::Verbose);

					frameBinTime = frameBinTime + _frameList._frames[n]._fracExp*((*(lc->_gtiList))[i].stop()-_frameList._frameList[n].start())*area;
					frameBinTimeNoArea = frameBinTimeNoArea + _frameList._frames[n]._fracExp*((*(lc->_gtiList))[i].stop()-_frameList._frameList[n].start());

					nextGTI = true;
					n--;
				}
			}
			else if  ( _frameList._frameList[n].start() >= (*(lc->_gtiList))[i].stop() )
			{
				errstr << "Frame in next bin. " << Sas::message(Msg::Lib, Msg::Verbose);
				nextGTI = true;
				n--;
			}
			if (checkFrameAgain == false)
				n++;
		} // Frame while
		int bin = lc->cell((((*(lc->_gtiList))[i].stop() + (*(lc->_gtiList))[i].start())/2.0));
		frameTime[bin] += frameBinTime;
		frameTimeNoArea[bin] += frameBinTimeNoArea;
		gtiTime[bin] += ((*(lc->_gtiList))[i].stop() - (*(lc->_gtiList))[i].start());
		i++;
	} // GTI while

	for (unsigned int i = 0 ; i < frameTime.size() ; i++ )
	{
	  	if( gtiTime[i] != 0.0)
	  	{
	  		(*_expoCorr)[i] = (*_expoCorr)[i] + frameTime[i]/lc->getBinSize(i);
	  	}
		lc->_fracExp[i] =  frameTime[i]/lc->getBinSize(i);
		lc->_fracExpNoArea[i] = frameTimeNoArea[i]/lc->getBinSize(i);
		(*(lc->_effBinTime))[i] = frameTime[i];

	}
		//delete _expoCorr;
}


void Exposure::printCorr()
{

  for (unsigned int i = 0 ; i < _expoCorr->size() ; i++ )
    errstr << "ExpoCorr["<<i<<"]= "<<  (*_expoCorr)[i] << Sas::message(Msg::Lib, Msg::Sparse);
}


